@@ -27,7 +27,9 @@ namespace OpenBve {
 		internal static Fog PreviousFog = new Fog(0.0f, 0.0f, new World.ColorRGB(128, 128, 128), 0.0);
 		internal static Fog CurrentFog = new Fog(0.0f, 0.0f, new World.ColorRGB(128, 128, 128), 0.5);
 		internal static Fog NextFog = new Fog(0.0f, 0.0f, new World.ColorRGB(128, 128, 128), 1.0);
-
+		internal static float NoFogStart = 800.0f;
+		internal static float NoFogEnd = 1600.0f;
+		
 		// route constants
 		internal static string RouteComment = "";
 		internal static string RouteImage = "";
@@ -46,7 +48,7 @@ namespace OpenBve {
 		internal const double TemperatureLapseRate = -0.0065;
 		internal const double CoefficientOfStiffness = 144117.325646911;
 
-		// athmospheric functions
+		// atmospheric functions
 		internal static void CalculateSeaLevelConstants() {
 			RouteSeaLevelAirTemperature = RouteInitialAirTemperature - TemperatureLapseRate * RouteInitialElevation;
 			double Exponent = RouteAccelerationDueToGravity * MolarMass / (UniversalGasConstant * TemperatureLapseRate);
@@ -62,13 +64,17 @@ namespace OpenBve {
 			double x = RouteSeaLevelAirTemperature + TemperatureLapseRate * Elevation;
 			if (x >= 1.0) {
 				return x;
-			} else return 1.0;
+			} else {
+				return 1.0;
+			}
 		}
 		internal static double GetAirDensity(double AirPressure, double AirTemperature) {
 			double x = AirPressure * MolarMass / (UniversalGasConstant * AirTemperature);
 			if (x >= 0.001) {
 				return x;
-			} else return 0.001;
+			} else {
+				return 0.001;
+			}
 		}
 		internal static double GetAirPressure(double Elevation, double AirTemperature) {
 			double Exponent = -RouteAccelerationDueToGravity * MolarMass / (UniversalGasConstant * TemperatureLapseRate);
@@ -77,40 +83,42 @@ namespace OpenBve {
 				double x = RouteSeaLevelAirPressure * Math.Pow(Base, Exponent);
 				if (x >= 0.001) {
 					return x;
-				} return 0.001;
-			} else return 0.001;
+				} else {
+					return 0.001;
+				}
+			} else {
+				return 0.001;
+			}
 		}
 		internal static double GetSpeedOfSound(double AirPressure, double AirTemperature) {
 			double AirDensity = GetAirDensity(AirPressure, AirTemperature);
 			return Math.Sqrt(CoefficientOfStiffness / AirDensity);
 		}
+		internal static double GetSpeedOfSound(double AirDensity) {
+			return Math.Sqrt(CoefficientOfStiffness / AirDensity);
+		}
 
-		// game constants
-		internal static int PretrainsUsed = 0;
-		internal static double PretrainInterval = 120.0;
+		// other trains
+		internal static double[] PrecedingTrainTimeDeltas = new double[] { };
+		internal static BogusPretrainInstruction[] BogusPretrainInstructions = new BogusPretrainInstruction[] { };
 
 		// startup
 		internal enum TrainStartMode {
 			ServiceBrakesAts = -1,
-			EmergencyBrakesAts = 0,
-			EmergencyBrakesNoAts = 1
+			EmergencyBrakesNoAts = 0,
+			EmergencyBrakesAts = 1
 		}
 		internal static TrainStartMode TrainStart = TrainStartMode.EmergencyBrakesNoAts;
 		internal static string TrainName = "";
 
 		// information
-		internal enum OutputMode {
-			Default = 0,
-			Fps = 1,
-			Debug = 2,
-			None = 3
-		}
 		internal static double InfoFrameRate = 1.0;
-		internal static int InfoTexturesRegistered = 0;
-		internal static int InfoTexturesLoaded = 0;
-		internal static int InfoSoundSourcesRegistered = 0;
-		internal static int InfoSoundSourcesPlaying = 0;
 		internal static string InfoDebugString = "";
+		internal static int InfoTotalTriangles = 0;
+		internal static int InfoTotalTriangleStrip = 0;
+		internal static int InfoTotalQuadStrip = 0;
+		internal static int InfoTotalQuads = 0;
+		internal static int InfoTotalPolygon = 0;
 
 		// ================================
 
@@ -167,14 +175,14 @@ namespace OpenBve {
 				// full menu
 				int n = 0;
 				for (int i = 0; i < Stations.Length; i++) {
-					if (Stations[i].StopAtStation & Stations[i].Stops.Length > 0) {
+					if (PlayerStopsAtStation(i) & Stations[i].Stops.Length > 0) {
 						n++;
 					}
 				}
 				MenuEntry[] a = new MenuCommand[n];
 				n = 0;
 				for (int i = 0; i < Stations.Length; i++) {
-					if (Stations[i].StopAtStation & Stations[i].Stops.Length > 0) {
+					if (PlayerStopsAtStation(i) & Stations[i].Stops.Length > 0) {
 						a[n] = new MenuCommand(Stations[i].Name, MenuTag.JumpToStation, i);
 						n++;
 					}
@@ -238,12 +246,20 @@ namespace OpenBve {
 			Messages = new Message[] { };
 			MarkerTextures = new int[] { };
 			PointsOfInterest = new PointOfInterest[] { };
+			PrecedingTrainTimeDeltas = new double[] { };
 			BogusPretrainInstructions = new BogusPretrainInstruction[] { };
 			TrainName = "";
 			TrainStart = TrainStartMode.EmergencyBrakesNoAts;
 			PreviousFog = new Fog(0.0f, 0.0f, new World.ColorRGB(128, 128, 128), 0.0);
 			CurrentFog = new Fog(0.0f, 0.0f, new World.ColorRGB(128, 128, 128), 0.5);
 			NextFog = new Fog(0.0f, 0.0f, new World.ColorRGB(128, 128, 128), 1.0);
+			NoFogStart = (float)World.BackgroundImageDistance + 200.0f;
+			NoFogEnd = 2.0f * NoFogStart;
+			InfoTotalTriangles = 0;
+			InfoTotalTriangleStrip = 0;
+			InfoTotalQuads = 0;
+			InfoTotalQuadStrip = 0;
+			InfoTotalPolygon = 0;
 			if (ResetLogs) {
 				LogRouteName = "";
 				LogTrainName = "";
@@ -718,6 +734,12 @@ namespace OpenBve {
 			Ats = 0,
 			Atc = 1
 		}
+		internal enum StationStopMode {
+			AllStop = 0,
+			AllPass = 1,
+			PlayerStop = 2,
+			PlayerPass = 3
+		}
 		internal struct Station {
 			internal string Name;
 			internal double ArrivalTime;
@@ -726,7 +748,7 @@ namespace OpenBve {
 			internal int DepartureSoundIndex;
 			internal double StopTime;
 			internal World.Vector3D SoundOrigin;
-			internal bool StopAtStation;
+			internal StationStopMode StopMode;
 			internal bool IsTerminalStation;
 			internal bool ForceStopSignal;
 			internal bool OpenLeftDoors;
@@ -749,6 +771,18 @@ namespace OpenBve {
 				return Stations[StationIndex].Stops.Length - 1;
 			} else return j;
 		}
+		/// <summary>Indicates whether the player's train stops at a station.</summary>
+		internal static bool PlayerStopsAtStation(int StationIndex) {
+			return Stations[StationIndex].StopMode == StationStopMode.AllStop | Stations[StationIndex].StopMode == StationStopMode.PlayerStop;
+		}
+		/// <summary>Indicates whether a train stops at a station.</summary>
+		internal static bool StopsAtStation(int StationIndex, TrainManager.Train Train) {
+			if (Train == TrainManager.PlayerTrain) {
+				return Stations[StationIndex].StopMode == StationStopMode.AllStop | Stations[StationIndex].StopMode == StationStopMode.PlayerStop;
+			} else {
+				return Stations[StationIndex].StopMode == StationStopMode.AllStop | Stations[StationIndex].StopMode == StationStopMode.PlayerPass;
+			}
+		}
 
 		// ================================
 
@@ -797,8 +831,28 @@ namespace OpenBve {
 			}
 			internal bool Exists(TrainManager.Train Train) {
 				for (int i = 0; i < this.Trains.Length; i++) {
-					if (this.Trains[i] == Train) return true;
-				} return false;
+					if (this.Trains[i] == Train)
+						return true;
+				}
+				return false;
+			}
+			internal bool IsFree() {
+				for (int i = 0; i < this.Trains.Length; i++) {
+					if (this.Trains[i].State == TrainManager.TrainState.Available | this.Trains[i].State == TrainManager.TrainState.Bogus) {
+						return false;
+					}
+				}
+				return true;
+			}
+			internal TrainManager.Train GetFirstTrain(bool AllowBogusTrain) {
+				for (int i = 0; i < this.Trains.Length; i++) {
+					if (this.Trains[i].State == TrainManager.TrainState.Available) {
+						return this.Trains[i];
+					} else if (AllowBogusTrain & this.Trains[i].State == TrainManager.TrainState.Bogus) {
+						return this.Trains[i];
+					}
+				}
+				return null;
 			}
 		}
 		internal static Section[] Sections = new Section[] { };
@@ -833,20 +887,22 @@ namespace OpenBve {
 				TrainManager.Train train = null;
 				while (true) {
 					if (l >= 0) {
-						if (Sections[l].Trains.Length != 0) {
-							train = Sections[l].Trains[0];
+						train = Sections[l].GetFirstTrain(false);
+						if (train != null) {
 							break;
 						} else {
 							l = Sections[l].PreviousSection;
 						}
-					} else break;
+					} else {
+						break;
+					}
 				}
 				if (train == null) {
 					double b = -double.MaxValue;
 					for (int i = 0; i < TrainManager.Trains.Length; i++) {
-						if (!TrainManager.Trains[i].Disposed) {
-							if (TrainManager.Trains[i].PretrainAheadTimetable > b) {
-								b = TrainManager.Trains[i].PretrainAheadTimetable;
+						if (TrainManager.Trains[i].State == TrainManager.TrainState.Available) {
+							if (TrainManager.Trains[i].TimetableDelta > b) {
+								b = TrainManager.Trains[i].TimetableDelta;
 								train = TrainManager.Trains[i];
 							}
 						}
@@ -876,7 +932,7 @@ namespace OpenBve {
 					}
 					if (train == TrainManager.PlayerTrain & Stations[d].IsTerminalStation) {
 						settored = true;
-					} else if (t >= 0.0 & SecondsSinceMidnight < t - train.PretrainAheadTimetable) {
+					} else if (t >= 0.0 & SecondsSinceMidnight < t - train.TimetableDelta) {
 						settored = true;
 					} else if (!Sections[SectionIndex].TrainReachedStopPoint) {
 						settored = true;
@@ -886,7 +942,7 @@ namespace OpenBve {
 				}
 			}
 			// train in block
-			if (Sections[SectionIndex].Trains.Length != 0) {
+			if (!Sections[SectionIndex].IsFree()) {
 				settored = true;
 			}
 			// free sections
@@ -973,11 +1029,17 @@ namespace OpenBve {
 			private double CurrentInterval;
 			private bool BrakeMode;
 			private double PersonalitySpeedFactor;
+			private int PowerNotchAtWhichWheelSlipIsObserved;
+			private double AtsChimeCancelPosition;
+			private int AtsChimeCancelSection;
 			internal SimplisticHumanDriverAI(TrainManager.Train Train) {
 				this.TimeLastProcessed = 0.0;
 				this.CurrentInterval = 1.0;
 				this.BrakeMode = false;
-				this.PersonalitySpeedFactor = 0.8 + 0.2 * Generator.NextDouble();
+				this.PersonalitySpeedFactor = 0.92 + 0.08 * Generator.NextDouble();
+				this.PowerNotchAtWhichWheelSlipIsObserved = Train.Specs.MaximumPowerNotch + 1;
+				this.AtsChimeCancelPosition = Train.Cars[0].FrontAxle.Follower.TrackPosition + 600.0;
+				this.AtsChimeCancelSection = -1;
 			}
 			internal override void Trigger(TrainManager.Train Train) {
 				if (SecondsSinceMidnight - TimeLastProcessed >= CurrentInterval) {
@@ -987,34 +1049,95 @@ namespace OpenBve {
 					for (int i = 0; i < Train.Cars.Length; i++) {
 						for (int j = 0; j < Train.Cars[i].Specs.Doors.Length; j++) {
 							if (Train.Cars[i].Specs.Doors[j].State != 0.0) {
-								doorsopen = true; break;
-							} if (doorsopen) break;
+								doorsopen = true;
+								break;
+							}
+							if (doorsopen) break;
 						}
 					}
 					// handle the security system
 					if (Train.Specs.Security.Mode == TrainManager.SecuritySystem.None) {
+						// none
 						Train.Specs.Security.ModeChange = TrainManager.SecuritySystem.AtsSN;
+						this.AtsChimeCancelPosition = Train.Cars[0].FrontAxle.Follower.TrackPosition + 600.0;
+						this.AtsChimeCancelSection = -1;
 					} else if (Train.Specs.Security.Mode == TrainManager.SecuritySystem.AtsSN) {
 						// ats-s
 						if (Train.Specs.Security.State == TrainManager.SecurityState.Ringing) {
-							TrainManager.AcknowledgeSecuritySystem(Train, TrainManager.AcknowledgementType.Alarm);
+							if (Train.Specs.CurrentPowerNotch.Driver == 0 & (Train.Specs.CurrentBrakeNotch.Driver >= 1 & Train.Cars[Train.DriverCar].Specs.BrakeType != TrainManager.CarBrakeType.AutomaticAirBrake | Train.Specs.AirBrake.Handle.Driver == TrainManager.AirBrakeHandleState.Service & Train.Cars[Train.DriverCar].Specs.BrakeType != TrainManager.CarBrakeType.AutomaticAirBrake) | Train.Specs.CurrentEmergencyBrake.Driver) {
+								// acknowledge bell
+								TrainManager.AcknowledgeSecuritySystem(Train, TrainManager.AcknowledgementType.Alarm);
+								// keep track of next red signal
+								const double lookahead = 700.0;
+								double tp = Train.Cars[0].FrontAxle.Follower.TrackPosition - Train.Cars[0].FrontAxlePosition + 0.5 * Train.Cars[0].Length;
+								{
+									int te = Train.Cars[0].FrontAxle.Follower.LastTrackElement;
+									for (int i = te; i < TrackManager.CurrentTrack.Elements.Length; i++) {
+										double stp = TrackManager.CurrentTrack.Elements[i].StartingTrackPosition;
+										if (tp + lookahead <= stp) break;
+										for (int j = 0; j < TrackManager.CurrentTrack.Elements[i].Events.Length; j++) {
+											if (TrackManager.CurrentTrack.Elements[i].Events[j] is TrackManager.SectionChangeEvent) {
+												if (Train.Specs.Security.Mode != TrainManager.SecuritySystem.Atc) {
+													TrackManager.SectionChangeEvent e = (TrackManager.SectionChangeEvent)TrackManager.CurrentTrack.Elements[i].Events[j];
+													if (stp + e.TrackPositionDelta > tp) {
+														if (!Game.Sections[e.NextSectionIndex].Invisible & Game.Sections[e.NextSectionIndex].CurrentAspect >= 0) {
+															double elim = Game.Sections[e.NextSectionIndex].Aspects[Game.Sections[e.NextSectionIndex].CurrentAspect].Speed;
+															if (elim == 0.0) {
+																if (this.AtsChimeCancelPosition > Game.Sections[e.NextSectionIndex].TrackPosition) {
+																	this.AtsChimeCancelPosition = Game.Sections[e.NextSectionIndex].TrackPosition;
+																	this.AtsChimeCancelSection = e.NextSectionIndex;
+																}
+															}
+														}
+													}
+												}
+											}
+										}
+									}
+								}
+								if (this.AtsChimeCancelPosition > tp + lookahead) {
+									this.AtsChimeCancelPosition = tp + lookahead;
+									this.AtsChimeCancelSection = -1;
+								}
+							}
 						} else if (Train.Specs.Security.State == TrainManager.SecurityState.Emergency & Math.Abs(Train.Specs.CurrentAverageSpeed) < 1.0) {
+							// reset the safety system
 							Train.Specs.Security.ModeChange = TrainManager.SecuritySystem.None;
 							this.CurrentInterval = 1.0;
 							return;
 						}
-						if (Math.Abs(Train.Specs.CurrentAverageSpeed) < 0.277777777777778) {
+						// cancel chime
+						if (Train.Cars[0].FrontAxle.Follower.TrackPosition > this.AtsChimeCancelPosition) {
 							TrainManager.AcknowledgeSecuritySystem(Train, TrainManager.AcknowledgementType.Chime);
+							this.AtsChimeCancelPosition = double.PositiveInfinity;
+							this.AtsChimeCancelSection = -1;
+						} else if (this.AtsChimeCancelSection >= 0 && Game.Sections[this.AtsChimeCancelSection].Aspects[Game.Sections[this.AtsChimeCancelSection].CurrentAspect].Speed != 0.0) {
+							TrainManager.AcknowledgeSecuritySystem(Train, TrainManager.AcknowledgementType.Chime);
+							this.AtsChimeCancelPosition = double.PositiveInfinity;
+							this.AtsChimeCancelSection = -1;
 						}
 					} else if (Train.Specs.Security.Mode == TrainManager.SecuritySystem.AtsP) {
 						// ats-p
-						if (Train.Specs.Security.State == TrainManager.SecurityState.Service) {
-							if (Math.Abs(Train.Specs.CurrentAverageSpeed) < 0.138888888888889) {
-								Train.Specs.Security.ModeChange = TrainManager.SecuritySystem.None;
-								this.CurrentInterval = 1.0;
-								return;
+						TrainManager.AcknowledgeSecuritySystem(Train, TrainManager.AcknowledgementType.Chime);
+						this.AtsChimeCancelPosition = double.PositiveInfinity;
+						this.AtsChimeCancelSection = -1;
+						if (Train.Specs.Security.State == TrainManager.SecurityState.Service & !Train.Specs.Security.Ats.AtsPOverride) {
+							if (Train.Specs.Security.Ats.AtsPDistance > 50.0 | Train.Station >= 0) {
+								if (Math.Abs(Train.Specs.CurrentAverageSpeed) < 4.16666666666667) {
+									TrainManager.AcknowledgeSecuritySystem(Train, TrainManager.AcknowledgementType.Override);
+									return;
+								}
+							} else {
+								if (Math.Abs(Train.Specs.CurrentAverageSpeed) < 0.277777777777778) {
+									Train.Specs.Security.ModeChange = TrainManager.SecuritySystem.None;
+									return;
+								}
 							}
 						}
+					} else {
+						TrainManager.AcknowledgeSecuritySystem(Train, TrainManager.AcknowledgementType.Chime);
+						this.AtsChimeCancelPosition = double.PositiveInfinity;
+						this.AtsChimeCancelSection = -1;
 					}
 					if (Train.Specs.Security.Eb.BellState != TrainManager.SecurityState.Normal) {
 						// eb
@@ -1023,23 +1146,41 @@ namespace OpenBve {
 					// do the ai
 					Train.Specs.CurrentConstSpeed = false;
 					TrainManager.ApplyHoldBrake(Train, false);
+					int stopIndex = Train.Station >= 0 ? GetStopIndex(Train.Station, Train.Cars.Length) : -1;
 					if (Train.CurrentSectionLimit == 0.0 & Train.Specs.Security.Mode != TrainManager.SecuritySystem.Atc) {
 						// passing red signal
 						TrainManager.ApplyEmergencyBrake(Train);
 						TrainManager.ApplyNotch(Train, -1, true, 1, true);
 						CurrentInterval = 0.5;
-					} else if (doorsopen) {
-						// door opened
+					} else if (doorsopen | Train.StationState == TrainManager.TrainStopState.Boarding) {
+						// door opened or boarding at station
+						this.PowerNotchAtWhichWheelSlipIsObserved = Train.Specs.MaximumPowerNotch + 1;
 						if (Train.Station >= 0 && Stations[Train.Station].IsTerminalStation & Train == TrainManager.PlayerTrain) {
 							// player's terminal station
 							TrainManager.ApplyReverser(Train, 0, false);
 							TrainManager.ApplyNotch(Train, -1, true, 1, true);
 							TrainManager.ApplyAirBrakeHandle(Train, TrainManager.AirBrakeHandleState.Service);
 							TrainManager.ApplyEmergencyBrake(Train);
+							Train.Specs.Security.ModeChange = TrainManager.SecuritySystem.None;
 							CurrentInterval = 10.0;
 						} else {
-							TrainManager.ApplyNotch(Train, -1, true, 1, true);
-							TrainManager.ApplyAirBrakeHandle(Train, TrainManager.AirBrakeHandleState.Service);
+							TrainManager.ApplyNotch(Train, -1, true, 0, true);
+							if (Train.Cars[Train.DriverCar].Specs.BrakeType == TrainManager.CarBrakeType.AutomaticAirBrake) {
+								if (Train.Cars[Train.DriverCar].Specs.AirBrake.BrakeCylinderCurrentPressure < 0.3 * Train.Cars[Train.DriverCar].Specs.AirBrake.BrakeCylinderServiceMaximumPressure) {
+									TrainManager.ApplyAirBrakeHandle(Train, TrainManager.AirBrakeHandleState.Service);
+								} else if (Train.Cars[Train.DriverCar].Specs.AirBrake.BrakeCylinderCurrentPressure > 0.9 * Train.Cars[Train.DriverCar].Specs.AirBrake.BrakeCylinderEmergencyMaximumPressure) {
+									TrainManager.ApplyAirBrakeHandle(Train, TrainManager.AirBrakeHandleState.Release);
+								} else {
+									TrainManager.ApplyAirBrakeHandle(Train, TrainManager.AirBrakeHandleState.Lap);
+								}
+							} else {
+								int b = (int)Math.Ceiling(0.5 * (double)Train.Specs.MaximumBrakeNotch);
+								if (Train.Specs.CurrentBrakeNotch.Driver < b) {
+									TrainManager.ApplyNotch(Train, 0, true, 1, true);
+								} else if (Train.Specs.CurrentBrakeNotch.Driver > b) {
+									TrainManager.ApplyNotch(Train, 0, true, -1, true);
+								}
+							}
 							TrainManager.UnapplyEmergencyBrake(Train);
 							if (Train.Station >= 0 & Train.StationState == TrainManager.TrainStopState.Completed) {
 								// ready for departure - close doors
@@ -1061,7 +1202,7 @@ namespace OpenBve {
 							}
 							CurrentInterval = 1.0;
 						}
-					} else if (Train.Station >= 0 && (Stations[Train.Station].StopAtStation & (Stations[Train.Station].OpenLeftDoors | Stations[Train.Station].OpenRightDoors) & Math.Abs(Train.Specs.CurrentAverageSpeed) < 0.25 & Train.StationState == TrainManager.TrainStopState.Pending)) {
+					} else if (Train.Station >= 0 && stopIndex >= 0 && Train.StationDistanceToStopPoint < Stations[Train.Station].Stops[stopIndex].BackwardTolerance && (StopsAtStation(Train.Station, Train) & (Stations[Train.Station].OpenLeftDoors | Stations[Train.Station].OpenRightDoors) & Math.Abs(Train.Specs.CurrentAverageSpeed) < 0.25 & Train.StationState == TrainManager.TrainStopState.Pending)) {
 						// arrived at station - open doors
 						if (Train.Specs.DoorOpenMode != TrainManager.DoorMode.Automatic) {
 							TrainManager.OpenTrainDoors(Train, Stations[Train.Station].OpenLeftDoors, Stations[Train.Station].OpenRightDoors);
@@ -1071,29 +1212,64 @@ namespace OpenBve {
 						// drive
 						TrainManager.UnapplyEmergencyBrake(Train);
 						TrainManager.ApplyReverser(Train, 1, false);
+						if (Train.Cars[Train.DriverCar].FrontAxle.CurrentWheelSlip | Train.Cars[Train.DriverCar].RearAxle.CurrentWheelSlip) {
+							// react to wheel slip
+							if (Train.Specs.CurrentPowerNotch.Driver > 1) {
+								this.PowerNotchAtWhichWheelSlipIsObserved = Train.Specs.CurrentPowerNotch.Driver;
+								TrainManager.ApplyNotch(Train, -1, true, -1, true);
+								TrainManager.ApplyAirBrakeHandle(Train, TrainManager.AirBrakeHandleState.Release);
+								this.CurrentInterval = 2.5;
+								return;
+							}
+						}
+						// initialize
 						double spd = Train.Specs.CurrentAverageSpeed;
 						double acc = Train.Specs.CurrentAverageAcceleration;
 						double lim = double.PositiveInfinity;
 						if (Train.Specs.Security.Mode == TrainManager.SecuritySystem.Atc) {
-							if (Train.Specs.Security.Atc.SpeedRestriction < lim) lim = Train.Specs.Security.Atc.SpeedRestriction;
+							if (Train.Specs.Security.Atc.SpeedRestriction < lim) {
+								lim = Train.Specs.Security.Atc.SpeedRestriction;
+							}
 						} else {
-							if (Train.CurrentRouteLimit < lim) lim = Train.CurrentRouteLimit;
-							if (Train.CurrentSectionLimit < lim) lim = Train.CurrentSectionLimit;
+							if (Train.CurrentRouteLimit < lim) {
+								lim = Train.CurrentRouteLimit;
+							}
+							if (Train.CurrentSectionLimit < lim) {
+								lim = Train.CurrentSectionLimit;
+							}
+						}
+						double powerstart, powerend, brakestart;
+						if (double.IsPositiveInfinity(lim)) {
+							powerstart = lim;
+							powerend = lim;
+							brakestart = lim;
+						} else {
+							lim *= this.PersonalitySpeedFactor;
+							if (spd < 8.33333333333333) {
+								powerstart = 0.666666666666667 * lim;
+								powerend = 0.833333333333333 * lim;
+							} else {
+								powerstart = lim - 2.77777777777778;
+								powerend = lim - 1.38888888888889;
+							}
+							if (this.BrakeMode) {
+								brakestart = powerend;
+							} else {
+								brakestart = lim + 0.555555555555556;
+							}
 						}
-						if (lim != double.PositiveInfinity) lim *= this.PersonalitySpeedFactor;
-						double powerstart = Math.Max(0.95 * lim, lim - 1.3);
-						double powerend = Math.Max(0.98 * lim, lim - 0.3);
-						double brakestart = BrakeMode ? lim : Math.Max(Math.Min(1.1 * lim, lim + 1.3), lim + 0.3);
 						double dec = 0.0;
 						double dectol;
 						double BrakeDeceleration = Train.Cars[Train.DriverCar].Specs.BrakeDecelerationAtServiceMaximumPressure;
 						if (Train.Cars[Train.DriverCar].Specs.BrakeType == TrainManager.CarBrakeType.AutomaticAirBrake | Train.Specs.MaximumBrakeNotch <= 0) {
 							dectol = 0.1 * BrakeDeceleration;
+						} else if (Train.Specs.MaximumBrakeNotch <= 2) {
+							dectol = 0.3 * BrakeDeceleration;
 						} else {
-							dectol = BrakeDeceleration / (double)Train.Specs.MaximumBrakeNotch;
+							dectol = 0.9 * BrakeDeceleration / (double)Train.Specs.MaximumBrakeNotch;
 						}
 						if (spd > 0.0 & spd > brakestart) {
-							double r = 0.125 + (spd - lim) / spd;
+							double r = 0.125 + (spd - powerend) / spd;
 							if (r > 0.0) {
 								if (r > 1.0) {
 									dec = BrakeDeceleration;
@@ -1102,9 +1278,12 @@ namespace OpenBve {
 								}
 							}
 						}
-						double lookahead = 25.0 + 1.5 * (spd * spd) / (2.0 * BrakeDeceleration);
+						double cruisedec = 0.01 * BrakeDeceleration;
+						// lookahead
+						double lookahead = (Train.Station >= 0 ? 150.0 : 75.0) + (spd * spd) / BrakeDeceleration;
 						double tp = Train.Cars[0].FrontAxle.Follower.TrackPosition - Train.Cars[0].FrontAxlePosition + 0.5 * Train.Cars[0].Length;
-						{ // events
+						{
+							// events
 							int te = Train.Cars[0].FrontAxle.Follower.LastTrackElement;
 							for (int i = te; i < TrackManager.CurrentTrack.Elements.Length; i++) {
 								double stp = TrackManager.CurrentTrack.Elements[i].StartingTrackPosition;
@@ -1129,16 +1308,32 @@ namespace OpenBve {
 														double dist = stp + e.TrackPositionDelta - tp;
 														double edec;
 														if (elim == 0.0) {
-															const double redstopdist = 10.0;
+															double redstopdist;
+															if (Train.Station >= 0 & Train.StationState == TrainManager.TrainStopState.Completed & dist < 150.0) {
+																dist = 1.0;
+																redstopdist = 25.0;
+															} else if (Train.Station >= 0 & Train.StationState == TrainManager.TrainStopState.Pending) {
+																redstopdist = 1.0;
+															} else if (spd > 6.94444444444444) {
+																redstopdist = 85.0;
+															} else {
+																if (Train.Specs.Security.Mode == TrainManager.SecuritySystem.AtsP) {
+																	redstopdist = 35.0;
+																} else {
+																	redstopdist = 25.0;
+																}
+															}
 															if (dist > redstopdist) {
-																edec = (spd * spd - elim * elim) / (2.0 * (dist - redstopdist));
+																edec = (spd * spd) / (2.0 * (dist - redstopdist));
 															} else {
 																edec = BrakeDeceleration;
 															}
 														} else {
 															if (dist >= 1.0) {
 																edec = (spd * spd - elim * elim) / (2.0 * dist);
-															} else edec = 0.0;
+															} else {
+																edec = 0.0;
+															}
 														}
 														if (edec > dec) dec = edec;
 													}
@@ -1148,17 +1343,26 @@ namespace OpenBve {
 									} else if (TrackManager.CurrentTrack.Elements[i].Events[j] is TrackManager.StationEndEvent) {
 										// station stop
 										TrackManager.StationEndEvent e = (TrackManager.StationEndEvent)TrackManager.CurrentTrack.Elements[i].Events[j];
-										if (Stations[e.StationIndex].StopAtStation & (Stations[e.StationIndex].OpenLeftDoors | Stations[e.StationIndex].OpenRightDoors) & Train.StationState == TrainManager.TrainStopState.Pending) {
+										if (StopsAtStation(e.StationIndex, Train) & Train.StationState == TrainManager.TrainStopState.Pending) {
 											int s = GetStopIndex(e.StationIndex, Train.Cars.Length);
 											if (s >= 0) {
 												double dist = Stations[e.StationIndex].Stops[s].TrackPosition - tp;
-												double edec;
-												if (dist < Stations[e.StationIndex].Stops[s].BackwardTolerance & -dist < Stations[e.StationIndex].Stops[s].ForwardTolerance) {
-													edec = BrakeDeceleration;
-												} else {
-													edec = (spd * spd) / (2.0 * dist);
+												if (dist > -Stations[e.StationIndex].Stops[s].ForwardTolerance) {
+													double edec;
+													if (dist < 0.0 | dist < Stations[e.StationIndex].Stops[s].BackwardTolerance & spd < 0.277777777777778) {
+														edec = BrakeDeceleration;
+													} else {
+														if (spd > 4.16666666666667) {
+															if (dist > 15.0) {
+																dist -= 7.5;
+															} else if (dist > 0.0) {
+																dist *= 0.5;
+															}
+														}
+														edec = (spd * spd) / (2.0 * dist);
+													}
+													if (edec > dec) dec = edec;
 												}
-												if (edec > dec) dec = edec;
 											}
 										}
 									}
@@ -1167,15 +1371,24 @@ namespace OpenBve {
 						}
 						// current station
 						if (Train.Station >= 0) {
-							if (Stations[Train.Station].StopAtStation & (Stations[Train.Station].OpenLeftDoors | Stations[Train.Station].OpenRightDoors) & Train.StationState == TrainManager.TrainStopState.Pending) {
+							if (StopsAtStation(Train.Station, Train) & Train.StationState == TrainManager.TrainStopState.Pending) {
 								int s = GetStopIndex(Train.Station, Train.Cars.Length);
 								if (s >= 0) {
 									double dist = Stations[Train.Station].Stops[s].TrackPosition - tp;
 									double edec;
-									if (dist < Stations[Train.Station].Stops[s].BackwardTolerance) {
+									if (dist < 0.0) {
 										edec = BrakeDeceleration;
-									} else {
+									} else if (dist < Stations[Train.Station].Stops[s].BackwardTolerance) {
+										if (spd > 2.77777777777778) {
+											if (dist > 15.0) {
+												dist -= 7.5;
+											} else if (dist > 0.0) {
+												dist *= 0.5;
+											}
+										}
 										edec = (spd * spd) / (2.0 * dist);
+									} else {
+										edec = 0.0;
 									}
 									if (edec > dec) dec = edec;
 								}
@@ -1189,9 +1402,14 @@ namespace OpenBve {
 							}
 						} else if (Train.Specs.Security.Mode == TrainManager.SecuritySystem.AtsP) {
 							// ats-p
-							if (Train.Specs.Security.State == TrainManager.SecurityState.Pattern & !Train.Specs.Security.Ats.AtsPOverride | Train.Specs.Security.State == TrainManager.SecurityState.Service) {
+							if (Train.Specs.Security.State == TrainManager.SecurityState.Service & !Train.Specs.Security.Ats.AtsPOverride) {
 								powerstart = spd - 1.0;
 								powerend = spd - 1.0;
+							} else if (Train.Specs.Security.State == TrainManager.SecurityState.Pattern & !Train.Specs.Security.Ats.AtsPOverride) {
+								if (spd > 6.94444444444444) {
+									powerstart = spd - 1.0;
+									powerend = spd - 1.0;
+								}
 							}
 						}
 						// power / brake
@@ -1201,62 +1419,103 @@ namespace OpenBve {
 							double decdiff = -acc - dec;
 							if (decdiff < -dectol) {
 								// brake start
-								TrainManager.ApplyNotch(Train, -1, true, 1, true);
-								TrainManager.ApplyAirBrakeHandle(Train, TrainManager.AirBrakeHandleState.Service);
+								if (Train.Specs.CurrentPowerNotch.Driver == 0) {
+									TrainManager.ApplyNotch(Train, 0, true, 1, true);
+									TrainManager.ApplyAirBrakeHandle(Train, TrainManager.AirBrakeHandleState.Service);
+								} else {
+									TrainManager.ApplyNotch(Train, -1, true, 0, true);
+								}
 								CurrentInterval *= 0.3;
-								if (CurrentInterval < 0.25) CurrentInterval = 0.25;
+								if (CurrentInterval < 0.3) CurrentInterval = 0.3;
 							} else if (decdiff > dectol) {
 								// brake stop
 								TrainManager.ApplyNotch(Train, -1, true, -1, true);
 								TrainManager.ApplyAirBrakeHandle(Train, TrainManager.AirBrakeHandleState.Release);
 								CurrentInterval *= 0.3;
-								if (CurrentInterval < 0.25) CurrentInterval = 0.25;
+								if (CurrentInterval < 0.3) CurrentInterval = 0.3;
 							} else {
 								// keep brake
 								TrainManager.ApplyNotch(Train, -1, true, 0, true);
 								TrainManager.ApplyAirBrakeHandle(Train, TrainManager.AirBrakeHandleState.Lap);
 								CurrentInterval *= 1.3;
-								if (CurrentInterval > 2.5) CurrentInterval = 2.5;
+								if (CurrentInterval > 1.7) CurrentInterval = 1.7;
 							}
-							if (Train.Specs.HasHoldBrake) {
-								if (Train.Cars[Train.DriverCar].Specs.BrakeType == TrainManager.CarBrakeType.AutomaticAirBrake) {
-									if (Train.Specs.AirBrake.Handle.Driver == TrainManager.AirBrakeHandleState.Release) {
-										TrainManager.ApplyHoldBrake(Train, true);
-									}
-								} else if (Train.Specs.CurrentBrakeNotch.Driver == 0) {
-									TrainManager.ApplyHoldBrake(Train, true);
-								}
+							if (Train.Specs.CurrentPowerNotch.Driver == 0 & Train.Specs.CurrentBrakeNotch.Driver == 0) {
+								TrainManager.ApplyHoldBrake(Train, Train.Specs.HasHoldBrake);
 							}
 							if (Train.Cars[Train.DriverCar].Specs.BrakeType == TrainManager.CarBrakeType.AutomaticAirBrake) {
 								CurrentInterval = 0.1;
 							}
+						} else if (dec > cruisedec) {
+							// cut power and brake
+							BrakeMode = true;
+							TrainManager.ApplyNotch(Train, -1, true, -1, true);
+							TrainManager.ApplyAirBrakeHandle(Train, TrainManager.AirBrakeHandleState.Release);
+							if (Train.Specs.CurrentPowerNotch.Driver == 0 & Train.Specs.CurrentBrakeNotch.Driver == 0) {
+								TrainManager.ApplyHoldBrake(Train, Train.Specs.HasHoldBrake);
+							}
 						} else {
 							// power
 							BrakeMode = false;
+							double acclim;
+							if (!double.IsInfinity(lim)) {
+								double d = lim - spd;
+								if (d > 0.0) {
+									acclim = 0.1 / (0.1 * d + 1.0) - 0.12;
+								} else {
+									acclim = -1.0;
+								}
+							} else {
+								acclim = -1.0;
+							}
 							if (spd < powerstart) {
-								// power start
-								TrainManager.ApplyNotch(Train, 1, true, -1, true);
+								// power start (under-speed)
+								if (Train.Specs.CurrentBrakeNotch.Driver == 0) {
+									if (Train.Specs.CurrentPowerNotch.Driver < this.PowerNotchAtWhichWheelSlipIsObserved - 1) {
+										TrainManager.ApplyNotch(Train, 1, true, 0, true);
+									}
+								} else {
+									TrainManager.ApplyNotch(Train, 0, true, -1, true);
+								}
 								TrainManager.ApplyAirBrakeHandle(Train, TrainManager.AirBrakeHandleState.Release);
-								CurrentInterval *= 0.3;
-								if (CurrentInterval < 0.25) CurrentInterval = 0.25;
+								if (double.IsPositiveInfinity(powerstart)) {
+									CurrentInterval = 0.3 + 0.1 * Train.Specs.CurrentPowerNotch.Driver;
+								} else {
+									double p = (double)Train.Specs.CurrentPowerNotch.Driver / (double)Train.Specs.MaximumPowerNotch;
+									CurrentInterval = 0.3 + 15.0 * p / (powerstart - spd + 1.0);
+								}
+								if (CurrentInterval > 1.3) CurrentInterval = 1.3;
 							} else if (spd > powerend) {
-								// power end
+								// power end (over-speed)
 								TrainManager.ApplyNotch(Train, -1, true, -1, true);
 								TrainManager.ApplyAirBrakeHandle(Train, TrainManager.AirBrakeHandleState.Release);
 								CurrentInterval *= 0.3;
-								if (CurrentInterval < 0.25) CurrentInterval = 0.25;
+								if (CurrentInterval < 0.3) CurrentInterval = 0.3;
+							} else if (acc < acclim) {
+								// power start (under-acceleration)
+								if (Train.Specs.CurrentBrakeNotch.Driver == 0) {
+									if (Train.Specs.CurrentPowerNotch.Driver < this.PowerNotchAtWhichWheelSlipIsObserved - 1) {
+										if (Train.Specs.CurrentPowerNotch.Driver == Train.Specs.CurrentPowerNotch.Actual) {
+											TrainManager.ApplyNotch(Train, 1, true, 0, true);
+										}
+									}
+								} else {
+									TrainManager.ApplyNotch(Train, 0, true, -1, true);
+								}
+								TrainManager.ApplyAirBrakeHandle(Train, TrainManager.AirBrakeHandleState.Release);
+								CurrentInterval = 1.3;
 							} else {
 								// keep power
 								TrainManager.ApplyNotch(Train, 0, true, -1, true);
 								TrainManager.ApplyAirBrakeHandle(Train, TrainManager.AirBrakeHandleState.Release);
-								if (Train.Specs.CurrentPowerNotch.Driver > 0) {
+								if (Train.Specs.CurrentPowerNotch.Driver != 0) {
 									Train.Specs.CurrentConstSpeed = Train.Specs.HasConstSpeed;
 								}
 								if (Train.Specs.CurrentPowerNotch.Driver == 0 & Train.Specs.CurrentBrakeNotch.Driver == 0) {
 									TrainManager.ApplyHoldBrake(Train, Train.Specs.HasHoldBrake);
 								}
 								CurrentInterval *= 1.3;
-								if (CurrentInterval > 2.5) CurrentInterval = 2.5;
+								if (CurrentInterval > 1.7) CurrentInterval = 1.7;
 							}
 						}
 					}
@@ -1269,7 +1528,6 @@ namespace OpenBve {
 			internal double TrackPosition;
 			internal double Time;
 		}
-		internal static BogusPretrainInstruction[] BogusPretrainInstructions = new BogusPretrainInstruction[] { };
 		internal class BogusPretrainAI : GeneralAI {
 			private double TimeLastProcessed;
 			private double CurrentInterval;
@@ -1319,7 +1577,6 @@ namespace OpenBve {
 		// ================================
 
 		// messages
-		//internal enum MessageType { Game, Interface }
 		internal enum MessageColor {
 			None = 0,
 			Black = 1,
@@ -1466,10 +1723,6 @@ namespace OpenBve {
 			internal double TrackPitch;
 			internal double TrackRoll;
 			internal string Text;
-			//internal World.Vector3D Position;
-			//internal World.Vector3D Direction;
-			//internal World.Vector3D Up;
-			//internal World.Vector3D Side;
 		}
 		internal static PointOfInterest[] PointsOfInterest = new PointOfInterest[] { };
 		internal static bool ApplyPointOfView(int Value, bool Relative) {
@@ -1507,10 +1760,11 @@ namespace OpenBve {
 			// process poi
 			if (j >= 0) {
 				TrackManager.UpdateTrackFollower(ref World.CameraTrackFollower, t, true, false);
-				World.CameraCurrentAlignment.TrackOffset = PointsOfInterest[j].TrackOffset;
+				World.CameraCurrentAlignment.Position = PointsOfInterest[j].TrackOffset;
 				World.CameraCurrentAlignment.Yaw = PointsOfInterest[j].TrackYaw;
 				World.CameraCurrentAlignment.Pitch = PointsOfInterest[j].TrackPitch;
 				World.CameraCurrentAlignment.Roll = PointsOfInterest[j].TrackRoll;
+				World.CameraCurrentAlignment.TrackPosition = t;
 				World.UpdateAbsoluteCamera(0.0);
 				if (PointsOfInterest[j].Text != null) {
 					double n = 3.0 + 0.5 * Math.Sqrt((double)PointsOfInterest[j].Text.Length);
