@@ -6,30 +6,32 @@ using System.Windows.Forms;
 namespace OpenBve {
 	internal static class MainLoop {
 
-		//// declarations
+		// declarations
 		internal static bool LimitFramerate = false;
 		private static bool Quit = false;
+		private static int TimeFactor = 1;
+		private static ViewPortMode CurrentViewPortMode = ViewPortMode.Scenery;
 
-		//// --------------------------------
+		// --------------------------------
 
-		//// start loop
+		// start loop
 		internal static void StartLoop() {
 			// timer
 			Timers.Initialize();
-			// train loop sounds
-			for (int i = 0; i < TrainManager.Trains.Length; i++) {
-				for (int j = 0; j < TrainManager.Trains[i].Cars.Length; j++) {
-					if (TrainManager.Trains[i].Cars[j].Specs.IsMotorCar) {
-						if (TrainManager.Trains[i].Cars[j].Sounds.Loop.SoundBufferIndex >= 0) {
-							SoundManager.PlaySound(TrainManager.Trains[i].Cars[j].Sounds.Loop.SoundBufferIndex, TrainManager.Trains[i], j, TrainManager.Trains[i].Cars[j].Sounds.Loop.Position, SoundManager.Importance.AlwaysPlay, true);
-						}
-					}
-				}
-			}
 			// framerate display
 			double TotalTimeElapsedForInfo = 0.0;
 			double TotalTimeElapsedForSectionUpdate = 0.0;
 			int TotalFramesElapsed = 0;
+			// update sections
+			for (int i = 0; i < TrainManager.Trains.Length; i++) {
+				int s = TrainManager.Trains[i].Cars[0].CurrentSection;
+				if (s >= 0) {
+					Game.Sections[s].Enter(TrainManager.Trains[i]);
+				}
+			}
+			if (Game.Sections.Length > 0) {
+				Game.UpdateSection(Game.Sections.Length - 1);
+			}
 			// fast-forward until start time
 			{
 				Game.MinimalisticSimulation = true;
@@ -52,22 +54,17 @@ namespace OpenBve {
 				}
 				Game.MinimalisticSimulation = false;
 			}
-			// update sections
-			int s = TrainManager.PlayerTrain.Cars[0].CurrentSection;
-			if (s >= 0) {
-				Game.Sections[s].Enter(TrainManager.PlayerTrain);
-			}
-			if (Game.Sections.Length > 0) {
-				Game.UpdateSection(Game.Sections.Length - 1);
-			}
 			// loop
 			Asynchronous.Initialize();
 			World.InitializeCameraRestriction();
 			while (true) {
+				#if DEBUG
+				debugCheckError("MainLoop");
+				#endif
 				// timer
 				double TimeElapsed;
 				if (Game.SecondsSinceMidnight >= Game.StartupTime) {
-					TimeElapsed = Timers.GetElapsedTime();
+					TimeElapsed = Timers.GetElapsedTime() * (double)TimeFactor;
 				} else {
 					TimeElapsed = Game.StartupTime - Game.SecondsSinceMidnight;
 				}
@@ -80,34 +77,17 @@ namespace OpenBve {
 					}
 					TotalTimeElapsedForSectionUpdate = 0.0;
 				}
-				if (TotalTimeElapsedForInfo >= 0.25) {
+				if (TotalTimeElapsedForInfo >= 0.2) {
 					Game.InfoFrameRate = (double)TotalFramesElapsed / TotalTimeElapsedForInfo;
-					Game.InfoTexturesLoaded = 0;
-					Game.InfoTexturesRegistered = 0;
-					for (int i = 0; i < TextureManager.Textures.Length; i++) {
-						if (TextureManager.Textures[i] != null) {
-							if (TextureManager.Textures[i].Loaded) Game.InfoTexturesLoaded++;
-							Game.InfoTexturesRegistered++;
-						}
-					}
-					Game.InfoSoundSourcesRegistered = 0;
-					Game.InfoSoundSourcesPlaying = 0;
-					for (int i = 0; i < SoundManager.SoundSources.Length; i++) {
-						if (SoundManager.SoundSources[i] != null) {
-							Game.InfoSoundSourcesRegistered++;
-							if (!SoundManager.SoundSources[i].Suppressed) {
-								Game.InfoSoundSourcesPlaying++;
-							}
-						}
-					}
 					TotalTimeElapsedForInfo = 0.0;
 					TotalFramesElapsed = 0;
 				}
 				// events
 				ProcessEvents();
 				World.CameraAlignmentDirection = new World.CameraAlignment();
+				World.UpdateMouseGrab(TimeElapsed);
 				ProcessControls(TimeElapsed);
-				if (World.CameraMode == World.CameraViewMode.Interior) {
+				if (World.CameraMode == World.CameraViewMode.Interior | World.CameraMode == World.CameraViewMode.InteriorLookAhead) {
 					World.UpdateAbsoluteCamera(TimeElapsed);
 				}
 				if (Quit) break;
@@ -121,7 +101,7 @@ namespace OpenBve {
 						double a = World.CameraTrackFollower.TrackPosition;
 						TrainManager.UpdateTrains(v);
 						double b = World.CameraTrackFollower.TrackPosition;
-						if (World.CameraMode == World.CameraViewMode.Interior | World.CameraMode == World.CameraViewMode.Exterior) {
+						if (World.CameraMode == World.CameraViewMode.Interior | World.CameraMode == World.CameraViewMode.InteriorLookAhead | World.CameraMode == World.CameraViewMode.Exterior) {
 							World.CameraTrackFollower.TrackPosition = a;
 							TrackManager.UpdateTrackFollower(ref World.CameraTrackFollower, b, false, false);
 						}
@@ -130,14 +110,17 @@ namespace OpenBve {
 				}
 				// update in one piece
 				ObjectManager.UpdateAnimatedWorldObjects(TimeElapsed, false);
-				if (World.CameraMode == World.CameraViewMode.Interior | World.CameraMode == World.CameraViewMode.Exterior) {
-					ObjectManager.UpdateVisibility(World.CameraTrackFollower.TrackPosition + World.CameraCurrentAlignment.TrackOffset.Z);
+				if (World.CameraMode == World.CameraViewMode.Interior | World.CameraMode == World.CameraViewMode.InteriorLookAhead | World.CameraMode == World.CameraViewMode.Exterior) {
+					ObjectManager.UpdateVisibility(World.CameraTrackFollower.TrackPosition + World.CameraCurrentAlignment.Position.Z);
 					int d = TrainManager.PlayerTrain.DriverCar;
 					World.CameraSpeed = TrainManager.PlayerTrain.Cars[d].Specs.CurrentSpeed;
 				} else {
 					World.CameraSpeed = 0.0;
 				}
-				if (World.CameraMode != World.CameraViewMode.Interior) {
+				if (World.CameraRestriction == World.CameraRestrictionMode.NotAvailable) {
+					World.UpdateDriverBody(TimeElapsed);
+					World.UpdateAbsoluteCamera(TimeElapsed);
+				} else if (World.CameraMode != World.CameraViewMode.Interior) {
 					World.UpdateAbsoluteCamera(TimeElapsed);
 				}
 				Game.UpdateScore(TimeElapsed);
@@ -165,13 +148,17 @@ namespace OpenBve {
 					System.Threading.Thread.Sleep(10);
 				}
 			}
-			Interface.SaveLogs();
-			Interface.SaveOptions();
+			try {
+				Interface.SaveLogs();
+			} catch { }
+			try {
+				Interface.SaveOptions();
+			} catch { }
 		}
 
-		//// --------------------------------
+		// --------------------------------
 
-		//// process events
+		// process events
 		private static Interface.KeyboardModifier CurrentKeyboardModifier = Interface.KeyboardModifier.None;
 		private static void ProcessEvents() {
 			Sdl.SDL_Event Event;
@@ -185,7 +172,7 @@ namespace OpenBve {
 					case Sdl.SDL_VIDEORESIZE:
 						Renderer.ScreenWidth = Event.resize.w;
 						Renderer.ScreenHeight = Event.resize.h;
-						UpdateViewport();
+						UpdateViewport(MainLoop.ViewPortChangeMode.NoChange);
 						InitializeMotionBlur();
 						break;
 						// key down
@@ -323,11 +310,31 @@ namespace OpenBve {
 								}
 							}
 						} break;
+					case Sdl.SDL_MOUSEBUTTONDOWN:
+						// mouse button down
+						if (Event.button.button == Sdl.SDL_BUTTON_RIGHT) {
+							World.MouseGrabEnabled = !World.MouseGrabEnabled;
+							if (World.MouseGrabEnabled) {
+								World.MouseGrabTarget = new World.Vector2D(0.0, 0.0);
+								Sdl.SDL_WM_GrabInput(Sdl.SDL_GRAB_ON);
+								Game.AddMessage(Interface.GetInterfaceString("notification_mousegrab_on"), Game.MessageDependency.None, Interface.GameMode.Expert, Game.MessageColor.Blue, Game.SecondsSinceMidnight + 5.0);
+							} else {
+								Sdl.SDL_WM_GrabInput(Sdl.SDL_GRAB_OFF);
+								Game.AddMessage(Interface.GetInterfaceString("notification_mousegrab_off"), Game.MessageDependency.None, Interface.GameMode.Expert, Game.MessageColor.Blue, Game.SecondsSinceMidnight + 5.0);
+							}
+						}
+						break;
+					case Sdl.SDL_MOUSEMOTION:
+						// mouse motion
+						if (World.MouseGrabEnabled) {
+							World.MouseGrabTarget = new World.Vector2D((double)Event.motion.xrel, (double)Event.motion.yrel);
+						}
+						break;
 				}
 			}
 		}
 
-		//// process controls
+		// process controls
 		private static void ProcessControls(double TimeElapsed) {
 			switch (Game.CurrentInterface) {
 				case Game.InterfaceType.Pause:
@@ -348,6 +355,13 @@ namespace OpenBve {
 										Game.CreateMenu(true);
 										Game.CurrentInterface = Game.InterfaceType.Menu;
 										break;
+									case Interface.Command.MiscFullscreen:
+										ToggleFullscreen();
+										break;
+									case Interface.Command.MiscMute:
+										SoundManager.Mute = !SoundManager.Mute;
+										SoundManager.Update(0.0);
+										break;
 								}
 							}
 						}
@@ -491,6 +505,16 @@ namespace OpenBve {
 											Array.Resize<int>(ref Game.CurrentMenuSelection, Game.CurrentMenuSelection.Length - 1);
 											Array.Resize<double>(ref Game.CurrentMenuOffsets, Game.CurrentMenuOffsets.Length - 1);
 										} break;
+									case Interface.Command.MiscFullscreen:
+										// fullscreen
+										ToggleFullscreen();
+										break;
+									case Interface.Command.MiscMute:
+										// mute
+										SoundManager.Mute = !SoundManager.Mute;
+										SoundManager.Update(0.0);
+										break;
+
 								}
 							}
 						}
@@ -616,78 +640,78 @@ namespace OpenBve {
 										} break;
 									case Interface.Command.CameraMoveForward:
 										// camera move forward
-										if (World.CameraMode == World.CameraViewMode.Interior | World.CameraMode == World.CameraViewMode.Exterior) {
-											double s = World.CameraMode == World.CameraViewMode.Interior ? World.CameraInteriorTopSpeed : World.CameraExteriorTopSpeed;
-											World.CameraAlignmentDirection.TrackOffset.Z = s * Interface.CurrentControls[i].AnalogState;
+										if (World.CameraMode == World.CameraViewMode.Interior | World.CameraMode == World.CameraViewMode.InteriorLookAhead | World.CameraMode == World.CameraViewMode.Exterior) {
+											double s = World.CameraMode == World.CameraViewMode.Interior | World.CameraMode == World.CameraViewMode.InteriorLookAhead ? World.CameraInteriorTopSpeed : World.CameraExteriorTopSpeed;
+											World.CameraAlignmentDirection.Position.Z = s * Interface.CurrentControls[i].AnalogState;
 										} else {
 											World.CameraAlignmentDirection.TrackPosition = World.CameraExteriorTopSpeed * Interface.CurrentControls[i].AnalogState;
 										} break;
 									case Interface.Command.CameraMoveBackward:
 										// camera move backward
-										if (World.CameraMode == World.CameraViewMode.Interior | World.CameraMode == World.CameraViewMode.Exterior) {
-											double s = World.CameraMode == World.CameraViewMode.Interior ? World.CameraInteriorTopSpeed : World.CameraExteriorTopSpeed;
-											World.CameraAlignmentDirection.TrackOffset.Z = -s * Interface.CurrentControls[i].AnalogState;
+										if (World.CameraMode == World.CameraViewMode.Interior | World.CameraMode == World.CameraViewMode.InteriorLookAhead | World.CameraMode == World.CameraViewMode.Exterior) {
+											double s = World.CameraMode == World.CameraViewMode.Interior | World.CameraMode == World.CameraViewMode.InteriorLookAhead ? World.CameraInteriorTopSpeed : World.CameraExteriorTopSpeed;
+											World.CameraAlignmentDirection.Position.Z = -s * Interface.CurrentControls[i].AnalogState;
 										} else {
 											World.CameraAlignmentDirection.TrackPosition = -World.CameraExteriorTopSpeed * Interface.CurrentControls[i].AnalogState;
 										} break;
 									case Interface.Command.CameraMoveLeft:
 										// camera move left
 										{
-											double s = World.CameraMode == World.CameraViewMode.Interior ? World.CameraInteriorTopSpeed : World.CameraExteriorTopSpeed;
-											World.CameraAlignmentDirection.TrackOffset.X = -s * Interface.CurrentControls[i].AnalogState;
+											double s = World.CameraMode == World.CameraViewMode.Interior | World.CameraMode == World.CameraViewMode.InteriorLookAhead ? World.CameraInteriorTopSpeed : World.CameraExteriorTopSpeed;
+											World.CameraAlignmentDirection.Position.X = -s * Interface.CurrentControls[i].AnalogState;
 										} break;
 									case Interface.Command.CameraMoveRight:
 										// camera move right
 										{
-											double s = World.CameraMode == World.CameraViewMode.Interior ? World.CameraInteriorTopSpeed : World.CameraExteriorTopSpeed;
-											World.CameraAlignmentDirection.TrackOffset.X = s * Interface.CurrentControls[i].AnalogState;
+											double s = World.CameraMode == World.CameraViewMode.Interior | World.CameraMode == World.CameraViewMode.InteriorLookAhead ? World.CameraInteriorTopSpeed : World.CameraExteriorTopSpeed;
+											World.CameraAlignmentDirection.Position.X = s * Interface.CurrentControls[i].AnalogState;
 										} break;
 									case Interface.Command.CameraMoveUp:
 										// camera move up
 										{
-											double s = World.CameraMode == World.CameraViewMode.Interior ? World.CameraInteriorTopSpeed : World.CameraExteriorTopSpeed;
-											World.CameraAlignmentDirection.TrackOffset.Y = s * Interface.CurrentControls[i].AnalogState;
+											double s = World.CameraMode == World.CameraViewMode.Interior | World.CameraMode == World.CameraViewMode.InteriorLookAhead ? World.CameraInteriorTopSpeed : World.CameraExteriorTopSpeed;
+											World.CameraAlignmentDirection.Position.Y = s * Interface.CurrentControls[i].AnalogState;
 										} break;
 									case Interface.Command.CameraMoveDown:
 										// camera move down
 										{
-											double s = World.CameraMode == World.CameraViewMode.Interior ? World.CameraInteriorTopSpeed : World.CameraExteriorTopSpeed;
-											World.CameraAlignmentDirection.TrackOffset.Y = -s * Interface.CurrentControls[i].AnalogState;
+											double s = World.CameraMode == World.CameraViewMode.Interior | World.CameraMode == World.CameraViewMode.InteriorLookAhead ? World.CameraInteriorTopSpeed : World.CameraExteriorTopSpeed;
+											World.CameraAlignmentDirection.Position.Y = -s * Interface.CurrentControls[i].AnalogState;
 										} break;
 									case Interface.Command.CameraRotateLeft:
 										// camera rotate left
 										{
-											double s = World.CameraMode == World.CameraViewMode.Interior ? World.CameraInteriorTopAngularSpeed : World.CameraExteriorTopAngularSpeed;
+											double s = World.CameraMode == World.CameraViewMode.Interior | World.CameraMode == World.CameraViewMode.InteriorLookAhead ? World.CameraInteriorTopAngularSpeed : World.CameraExteriorTopAngularSpeed;
 											World.CameraAlignmentDirection.Yaw = -s * Interface.CurrentControls[i].AnalogState;
 										} break;
 									case Interface.Command.CameraRotateRight:
 										// camera rotate right
 										{
-											double s = World.CameraMode == World.CameraViewMode.Interior ? World.CameraInteriorTopAngularSpeed : World.CameraExteriorTopAngularSpeed;
+											double s = World.CameraMode == World.CameraViewMode.Interior | World.CameraMode == World.CameraViewMode.InteriorLookAhead ? World.CameraInteriorTopAngularSpeed : World.CameraExteriorTopAngularSpeed;
 											World.CameraAlignmentDirection.Yaw = s * Interface.CurrentControls[i].AnalogState;
 										} break;
 									case Interface.Command.CameraRotateUp:
 										// camera rotate up
 										{
-											double s = World.CameraMode == World.CameraViewMode.Interior ? World.CameraInteriorTopAngularSpeed : World.CameraExteriorTopAngularSpeed;
+											double s = World.CameraMode == World.CameraViewMode.Interior | World.CameraMode == World.CameraViewMode.InteriorLookAhead ? World.CameraInteriorTopAngularSpeed : World.CameraExteriorTopAngularSpeed;
 											World.CameraAlignmentDirection.Pitch = s * Interface.CurrentControls[i].AnalogState;
 										} break;
 									case Interface.Command.CameraRotateDown:
 										// camera rotate down
 										{
-											double s = World.CameraMode == World.CameraViewMode.Interior ? World.CameraInteriorTopAngularSpeed : World.CameraExteriorTopAngularSpeed;
+											double s = World.CameraMode == World.CameraViewMode.Interior | World.CameraMode == World.CameraViewMode.InteriorLookAhead ? World.CameraInteriorTopAngularSpeed : World.CameraExteriorTopAngularSpeed;
 											World.CameraAlignmentDirection.Pitch = -s * Interface.CurrentControls[i].AnalogState;
 										} break;
 									case Interface.Command.CameraRotateCCW:
 										// camera rotate ccw
-										if (World.CameraMode != World.CameraViewMode.Interior | !World.CameraRestriction) {
-											double s = World.CameraMode == World.CameraViewMode.Interior ? World.CameraInteriorTopAngularSpeed : World.CameraExteriorTopAngularSpeed;
+										if ((World.CameraMode != World.CameraViewMode.Interior & World.CameraMode != World.CameraViewMode.InteriorLookAhead) | World.CameraRestriction != World.CameraRestrictionMode.On) {
+											double s = World.CameraMode == World.CameraViewMode.Interior | World.CameraMode == World.CameraViewMode.InteriorLookAhead ? World.CameraInteriorTopAngularSpeed : World.CameraExteriorTopAngularSpeed;
 											World.CameraAlignmentDirection.Roll = -s * Interface.CurrentControls[i].AnalogState;
 										} break;
 									case Interface.Command.CameraRotateCW:
 										// camera rotate cw
-										if (World.CameraMode != World.CameraViewMode.Interior | !World.CameraRestriction) {
-											double s = World.CameraMode == World.CameraViewMode.Interior ? World.CameraInteriorTopAngularSpeed : World.CameraExteriorTopAngularSpeed;
+										if ((World.CameraMode != World.CameraViewMode.Interior & World.CameraMode != World.CameraViewMode.InteriorLookAhead) | World.CameraRestriction != World.CameraRestrictionMode.On) {
+											double s = World.CameraMode == World.CameraViewMode.Interior | World.CameraMode == World.CameraViewMode.InteriorLookAhead ? World.CameraInteriorTopAngularSpeed : World.CameraExteriorTopAngularSpeed;
 											World.CameraAlignmentDirection.Roll = s * Interface.CurrentControls[i].AnalogState;
 										} break;
 									case Interface.Command.CameraZoomIn:
@@ -726,9 +750,14 @@ namespace OpenBve {
 										break;
 									case Interface.Command.CameraInterior:
 										// camera: interior
-										Game.AddMessage(Interface.GetInterfaceString("notification_interior"), Game.MessageDependency.None, Interface.GameMode.Expert, Game.MessageColor.Blue, Game.SecondsSinceMidnight + 2.0);
 										SaveCameraSettings();
-										World.CameraMode = World.CameraViewMode.Interior;
+										if (World.CameraMode == World.CameraViewMode.Interior & World.CameraRestriction == World.CameraRestrictionMode.NotAvailable) {
+											World.CameraMode = World.CameraViewMode.InteriorLookAhead;
+											Game.AddMessage(Interface.GetInterfaceString("notification_interior_lookahead"), Game.MessageDependency.None, Interface.GameMode.Expert, Game.MessageColor.Blue, Game.SecondsSinceMidnight + 2.0);
+										} else {
+											World.CameraMode = World.CameraViewMode.Interior;
+											Game.AddMessage(Interface.GetInterfaceString("notification_interior"), Game.MessageDependency.None, Interface.GameMode.Expert, Game.MessageColor.Blue, Game.SecondsSinceMidnight + 2.0);
+										}
 										RestoreCameraSettings();
 										if (TrainManager.PlayerTrain.Cars.Length >= 1 && TrainManager.PlayerTrain.Cars[0].Sections.Length >= 1) {
 											TrainManager.ChangeCarSection(TrainManager.PlayerTrain, 0, 0);
@@ -740,11 +769,13 @@ namespace OpenBve {
 										}
 										World.CameraAlignmentDirection = new World.CameraAlignment();
 										World.CameraAlignmentSpeed = new World.CameraAlignment();
-										UpdateViewport();
+										UpdateViewport(MainLoop.ViewPortChangeMode.NoChange);
 										World.UpdateAbsoluteCamera(TimeElapsed);
 										World.UpdateViewingDistances();
-										if (!World.PerformCameraRestrictionTest()) {
-											World.InitializeCameraRestriction();
+										if (World.CameraRestriction != World.CameraRestrictionMode.NotAvailable) {
+											if (!World.PerformCameraRestrictionTest()) {
+												World.InitializeCameraRestriction();
+											}
 										}
 										break;
 									case Interface.Command.CameraExterior:
@@ -767,7 +798,7 @@ namespace OpenBve {
 										}
 										World.CameraAlignmentDirection = new World.CameraAlignment();
 										World.CameraAlignmentSpeed = new World.CameraAlignment();
-										UpdateViewport();
+										UpdateViewport(MainLoop.ViewPortChangeMode.NoChange);
 										World.UpdateAbsoluteCamera(TimeElapsed);
 										World.UpdateViewingDistances();
 										break;
@@ -803,7 +834,7 @@ namespace OpenBve {
 											}
 											World.CameraAlignmentDirection = new World.CameraAlignment();
 											World.CameraAlignmentSpeed = new World.CameraAlignment();
-											UpdateViewport();
+											UpdateViewport(ViewPortChangeMode.NoChange);
 											World.UpdateAbsoluteCamera(TimeElapsed);
 											World.UpdateViewingDistances();
 										} break;
@@ -814,8 +845,8 @@ namespace OpenBve {
 												World.CameraMode = World.CameraViewMode.Track;
 												Game.AddMessage(Interface.GetInterfaceString("notification_track"), Game.MessageDependency.None, Interface.GameMode.Expert, Game.MessageColor.Blue, Game.SecondsSinceMidnight + 2.0);
 											}
-											double z = World.CameraCurrentAlignment.TrackOffset.Z;
-											World.CameraCurrentAlignment.TrackOffset = new World.Vector3D(World.CameraCurrentAlignment.TrackOffset.X, World.CameraCurrentAlignment.TrackOffset.Y, 0.0);
+											double z = World.CameraCurrentAlignment.Position.Z;
+											World.CameraCurrentAlignment.Position = new World.Vector3D(World.CameraCurrentAlignment.Position.X, World.CameraCurrentAlignment.Position.Y, 0.0);
 											World.CameraCurrentAlignment.Zoom = 0.0;
 											World.CameraAlignmentDirection = new World.CameraAlignment();
 											World.CameraAlignmentSpeed = new World.CameraAlignment();
@@ -834,7 +865,7 @@ namespace OpenBve {
 											TrackManager.UpdateTrackFollower(ref World.CameraTrackFollower, World.CameraTrackFollower.TrackPosition + z, true, false);
 											World.CameraCurrentAlignment.TrackPosition = World.CameraTrackFollower.TrackPosition;
 											World.VerticalViewingAngle = World.OriginalVerticalViewingAngle;
-											UpdateViewport();
+											UpdateViewport(ViewPortChangeMode.NoChange);
 											World.UpdateAbsoluteCamera(TimeElapsed);
 											World.UpdateViewingDistances();
 										} break;
@@ -845,8 +876,8 @@ namespace OpenBve {
 												World.CameraMode = World.CameraViewMode.Track;
 												Game.AddMessage(Interface.GetInterfaceString("notification_track"), Game.MessageDependency.None, Interface.GameMode.Expert, Game.MessageColor.Blue, Game.SecondsSinceMidnight + 2.0);
 											}
-											double z = World.CameraCurrentAlignment.TrackOffset.Z;
-											World.CameraCurrentAlignment.TrackOffset = new World.Vector3D(World.CameraCurrentAlignment.TrackOffset.X, World.CameraCurrentAlignment.TrackOffset.Y, 0.0);
+											double z = World.CameraCurrentAlignment.Position.Z;
+											World.CameraCurrentAlignment.Position = new World.Vector3D(World.CameraCurrentAlignment.Position.X, World.CameraCurrentAlignment.Position.Y, 0.0);
 											World.CameraCurrentAlignment.Zoom = 0.0;
 											World.CameraAlignmentDirection = new World.CameraAlignment();
 											World.CameraAlignmentSpeed = new World.CameraAlignment();
@@ -865,14 +896,14 @@ namespace OpenBve {
 											TrackManager.UpdateTrackFollower(ref World.CameraTrackFollower, World.CameraTrackFollower.TrackPosition + z, true, false);
 											World.CameraCurrentAlignment.TrackPosition = World.CameraTrackFollower.TrackPosition;
 											World.VerticalViewingAngle = World.OriginalVerticalViewingAngle;
-											UpdateViewport();
+											UpdateViewport(ViewPortChangeMode.NoChange);
 											World.UpdateAbsoluteCamera(TimeElapsed);
 											World.UpdateViewingDistances();
 										} break;
 									case Interface.Command.CameraReset:
 										// camera: reset
-										if (World.CameraMode == World.CameraViewMode.Interior) {
-											World.CameraCurrentAlignment.TrackOffset = new World.Vector3D(0.0, 0.0, 0.0);
+										if (World.CameraMode == World.CameraViewMode.Interior | World.CameraMode == World.CameraViewMode.InteriorLookAhead) {
+											World.CameraCurrentAlignment.Position = new World.Vector3D(0.0, 0.0, 0.0);
 										}
 										World.CameraCurrentAlignment.Yaw = 0.0;
 										World.CameraCurrentAlignment.Pitch = 0.0;
@@ -881,9 +912,11 @@ namespace OpenBve {
 											TrackManager.UpdateTrackFollower(ref World.CameraTrackFollower, TrainManager.PlayerTrain.Cars[0].FrontAxle.Follower.TrackPosition, true, false);
 										} else if (World.CameraMode == World.CameraViewMode.FlyBy | World.CameraMode == World.CameraViewMode.FlyByZooming) {
 											if (TrainManager.PlayerTrain.Specs.CurrentAverageSpeed >= 0.0) {
-												TrackManager.UpdateTrackFollower(ref World.CameraTrackFollower, TrainManager.PlayerTrain.Cars[0].FrontAxle.Follower.TrackPosition + 50.0, true, false);
+												double d = 30.0 + TrainManager.PlayerTrain.Specs.CurrentAverageSpeed;
+												TrackManager.UpdateTrackFollower(ref World.CameraTrackFollower, TrainManager.PlayerTrain.Cars[0].FrontAxle.Follower.TrackPosition + d, true, false);
 											} else {
-												TrackManager.UpdateTrackFollower(ref World.CameraTrackFollower, TrainManager.PlayerTrain.Cars[TrainManager.PlayerTrain.Cars.Length - 1].RearAxle.Follower.TrackPosition - 50.0, true, false);
+												double d = 30.0 - TrainManager.PlayerTrain.Specs.CurrentAverageSpeed;
+												TrackManager.UpdateTrackFollower(ref World.CameraTrackFollower, TrainManager.PlayerTrain.Cars[TrainManager.PlayerTrain.Cars.Length - 1].RearAxle.Follower.TrackPosition - d, true, false);
 											}
 										}
 										World.CameraCurrentAlignment.TrackPosition = World.CameraTrackFollower.TrackPosition;
@@ -891,10 +924,10 @@ namespace OpenBve {
 										World.VerticalViewingAngle = World.OriginalVerticalViewingAngle;
 										World.CameraAlignmentDirection = new World.CameraAlignment();
 										World.CameraAlignmentSpeed = new World.CameraAlignment();
-										UpdateViewport();
+										UpdateViewport(ViewPortChangeMode.NoChange);
 										World.UpdateAbsoluteCamera(TimeElapsed);
 										World.UpdateViewingDistances();
-										if (World.CameraMode == World.CameraViewMode.Interior & World.CameraRestriction) {
+										if ((World.CameraMode == World.CameraViewMode.Interior | World.CameraMode == World.CameraViewMode.InteriorLookAhead) & World.CameraRestriction == World.CameraRestrictionMode.On) {
 											if (!World.PerformCameraRestrictionTest()) {
 												World.InitializeCameraRestriction();
 											}
@@ -902,9 +935,19 @@ namespace OpenBve {
 										break;
 									case Interface.Command.CameraRestriction:
 										// camera: restriction
-										World.CameraRestriction = !World.CameraRestriction;
-										World.InitializeCameraRestriction();
-										Game.AddMessage(Interface.GetInterfaceString(World.CameraRestriction ? "notification_camerarestriction_on" : "notification_camerarestriction_off"), Game.MessageDependency.None, Interface.GameMode.Expert, Game.MessageColor.Blue, Game.SecondsSinceMidnight + 2.0);
+										if (World.CameraRestriction != World.CameraRestrictionMode.NotAvailable) {
+											if (World.CameraRestriction == World.CameraRestrictionMode.Off) {
+												World.CameraRestriction = World.CameraRestrictionMode.On;
+											} else {
+												World.CameraRestriction = World.CameraRestrictionMode.Off;
+											}
+											World.InitializeCameraRestriction();
+											if (World.CameraRestriction == World.CameraRestrictionMode.Off) {
+												Game.AddMessage(Interface.GetInterfaceString("notification_camerarestriction_off"), Game.MessageDependency.None, Interface.GameMode.Expert, Game.MessageColor.Blue, Game.SecondsSinceMidnight + 2.0);
+											} else {
+												Game.AddMessage(Interface.GetInterfaceString("notification_camerarestriction_on"), Game.MessageDependency.None, Interface.GameMode.Expert, Game.MessageColor.Blue, Game.SecondsSinceMidnight + 2.0);
+											}
+										}
 										break;
 									case Interface.Command.SinglePower:
 										// single power
@@ -1301,6 +1344,15 @@ namespace OpenBve {
 										// clock
 										Renderer.OptionClock = !Renderer.OptionClock;
 										break;
+									case Interface.Command.MiscTimeFactor:
+										// time factor
+										if (Interface.CurrentOptions.GameMode == Interface.GameMode.Expert) {
+											Game.AddMessage(Interface.GetInterfaceString("notification_notavailableexpert"), Game.MessageDependency.None, Interface.GameMode.Expert, Game.MessageColor.Blue, Game.SecondsSinceMidnight + 5.0);
+										} else {
+											TimeFactor = TimeFactor == 1 ? 5 : 1;
+											Game.AddMessage(TimeFactor.ToString(System.Globalization.CultureInfo.InvariantCulture) + "x", Game.MessageDependency.None, Interface.GameMode.Expert, Game.MessageColor.Blue, Game.SecondsSinceMidnight + 5.0 * (double)TimeFactor);
+										}
+										break;
 									case Interface.Command.MiscSpeed:
 										// speed
 										if (Interface.CurrentOptions.GameMode == Interface.GameMode.Expert) {
@@ -1317,6 +1369,10 @@ namespace OpenBve {
 										// toggle fullscreen
 										ToggleFullscreen();
 										break;
+									case Interface.Command.MiscMute:
+										// mute
+										SoundManager.Mute = !SoundManager.Mute;
+										break;
 								}
 							} else if (Interface.CurrentControls[i].DigitalState == Interface.DigitalControlState.Released) {
 								// released
@@ -1377,12 +1433,13 @@ namespace OpenBve {
 			}
 		}
 		
-		//// --------------------------------
+		// --------------------------------
 
-		//// save camera setting
+		// save camera setting
 		private static void SaveCameraSettings() {
 			switch (World.CameraMode) {
 				case World.CameraViewMode.Interior:
+				case World.CameraViewMode.InteriorLookAhead:
 					World.CameraSavedInterior = World.CameraCurrentAlignment;
 					break;
 				case World.CameraViewMode.Exterior:
@@ -1392,15 +1449,15 @@ namespace OpenBve {
 				case World.CameraViewMode.FlyBy:
 				case World.CameraViewMode.FlyByZooming:
 					World.CameraSavedTrack = World.CameraCurrentAlignment;
-					World.CameraSavedTrackPosition = World.CameraTrackFollower.TrackPosition;
 					break;
 			}
 		}
 		
-		//// restore camera setting
+		// restore camera setting
 		private static void RestoreCameraSettings() {
 			switch (World.CameraMode) {
 				case World.CameraViewMode.Interior:
+				case World.CameraViewMode.InteriorLookAhead:
 					World.CameraCurrentAlignment = World.CameraSavedInterior;
 					break;
 				case World.CameraViewMode.Exterior:
@@ -1410,7 +1467,7 @@ namespace OpenBve {
 				case World.CameraViewMode.FlyBy:
 				case World.CameraViewMode.FlyByZooming:
 					World.CameraCurrentAlignment = World.CameraSavedTrack;
-					TrackManager.UpdateTrackFollower(ref World.CameraTrackFollower, World.CameraSavedTrackPosition, true, false);
+					TrackManager.UpdateTrackFollower(ref World.CameraTrackFollower, World.CameraSavedTrack.TrackPosition, true, false);
 					World.CameraCurrentAlignment.TrackPosition = World.CameraTrackFollower.TrackPosition;
 					break;
 			}
@@ -1418,9 +1475,9 @@ namespace OpenBve {
 			World.VerticalViewingAngle = World.OriginalVerticalViewingAngle;
 		}
 
-		//// --------------------------------
+		// --------------------------------
 
-		//// toggle fullscreen
+		// toggle fullscreen
 		internal static void ToggleFullscreen() {
 			Interface.CurrentOptions.FullscreenMode = !Interface.CurrentOptions.FullscreenMode;
 			Gl.glDisable(Gl.GL_FOG); Renderer.FogEnabled = false;
@@ -1437,27 +1494,45 @@ namespace OpenBve {
 				Renderer.ScreenHeight = Interface.CurrentOptions.WindowHeight;
 			}
 			Renderer.InitializeLighting();
-			UpdateViewport();
+			UpdateViewport(ViewPortChangeMode.NoChange);
 			InitializeMotionBlur();
 			Timetable.CreateTimetable();
 			Timetable.UpdateCustomTimetable(-1, -1);
 			World.InitializeCameraRestriction();
 		}
 
-		//// update viewport
-		internal static void UpdateViewport() {
+		// update viewport
+		internal enum ViewPortMode {
+			Scenery = 0,
+			Cab = 1
+		}
+		internal enum ViewPortChangeMode {
+			ChangeToScenery = 0,
+			ChangeToCab = 1,
+			NoChange = 2
+		}
+		internal static void UpdateViewport(ViewPortChangeMode Mode) {
+			if (Mode == ViewPortChangeMode.ChangeToCab) {
+				CurrentViewPortMode = ViewPortMode.Cab;
+			} else {
+				CurrentViewPortMode = ViewPortMode.Scenery;
+			}
 			Gl.glViewport(0, 0, Renderer.ScreenWidth, Renderer.ScreenHeight);
 			World.AspectRatio = (double)Renderer.ScreenWidth / (double)Renderer.ScreenHeight;
 			World.HorizontalViewingAngle = 2.0 * Math.Atan(Math.Tan(0.5 * World.VerticalViewingAngle) * World.AspectRatio);
 			Gl.glMatrixMode(Gl.GL_PROJECTION);
 			Gl.glLoadIdentity();
 			const double invdeg = 57.295779513082320877;
-			Glu.gluPerspective(World.VerticalViewingAngle * invdeg, -World.AspectRatio, 0.2, World.BackgroundImageDistance + 3.0 * World.ExtraViewingDistance);
+			if (CurrentViewPortMode == ViewPortMode.Cab) {
+				Glu.gluPerspective(World.VerticalViewingAngle * invdeg, -World.AspectRatio, 0.025, 50.0);
+			} else {
+				Glu.gluPerspective(World.VerticalViewingAngle * invdeg, -World.AspectRatio, 0.4, World.BackgroundImageDistance + 4.0 * World.ExtraViewingDistance);
+			}
 			Gl.glMatrixMode(Gl.GL_MODELVIEW);
 			Gl.glLoadIdentity();
 		}
 
-		//// initialize motion blur
+		// initialize motion blur
 		internal static void InitializeMotionBlur() {
 			if (Interface.CurrentOptions.MotionBlur != Interface.MotionBlurMode.None) {
 				if (Renderer.PixelBufferOpenGlTextureIndex != 0) {
@@ -1477,6 +1552,44 @@ namespace OpenBve {
 				Gl.glCopyTexImage2D(Gl.GL_TEXTURE_2D, 0, Gl.GL_RGB, 0, 0, w, h, 0);
 			}
 		}
+		
+		#if DEBUG
+		// check error
+		private static void debugCheckError(string Location) {
+			int error = Gl.glGetError();
+			if (error != Gl.GL_NO_ERROR) {
+				string message = Location + ": ";
+				switch (error) {
+					case Gl.GL_INVALID_ENUM:
+						message += "GL_INVALID_ENUM";
+						break;
+					case Gl.GL_INVALID_VALUE:
+						message += "GL_INVALID_VALUE";
+						break;
+					case Gl.GL_INVALID_OPERATION:
+						message += "GL_INVALID_OPERATION";
+						break;
+					case Gl.GL_STACK_OVERFLOW:
+						message += "GL_STACK_OVERFLOW";
+						break;
+					case Gl.GL_STACK_UNDERFLOW:
+						message += "GL_STACK_UNDERFLOW";
+						break;
+					case Gl.GL_OUT_OF_MEMORY:
+						message += "GL_OUT_OF_MEMORY";
+						break;
+					case Gl.GL_TABLE_TOO_LARGE:
+						message += "GL_TABLE_TOO_LARGE";
+						break;
+					default:
+						message += error.ToString();
+						break;
+				}
+				System.Windows.Forms.MessageBox.Show(message);
+				Quit = true;
+			}
+		}
+		#endif
 
 	}
 }
\ No newline at end of file
