@@ -205,22 +205,21 @@ namespace OpenBve {
 		
 		// initialize plugin
 		internal static void InitializePlugin(TrainManager.Train Train) {
-			if (Program.CurrentPlatform != Program.Platform.Windows) return;
 			if (!PluginLoaded) return;
 			switch (Game.TrainStart) {
 				case Game.TrainStartMode.ServiceBrakesAts:
 					PluginError = true;
-					Initialize(ATS_INIT_OFF);
+					Initialize(ATS_INIT_ON_SVC);
 					PluginError = false;
 					break;
 				case Game.TrainStartMode.EmergencyBrakesAts:
 					PluginError = true;
-					Initialize(ATS_INIT_ON_SVC);
+					Initialize(ATS_INIT_ON_EMG);
 					PluginError = false;
 					break;
 				case Game.TrainStartMode.EmergencyBrakesNoAts:
 					PluginError = true;
-					Initialize(ATS_INIT_ON_EMG);
+					Initialize(ATS_INIT_OFF);
 					PluginError = false;
 					break;
 				default:
@@ -233,7 +232,6 @@ namespace OpenBve {
 		
 		// unload plugin
 		internal static void UnloadPlugin() {
-			if (Program.CurrentPlatform != Program.Platform.Windows) return;
 			if (!PluginLoaded) return;
 			{
 				PluginError = true;
@@ -249,11 +247,10 @@ namespace OpenBve {
 		
 		// update plugin
 		internal static void UpdatePlugin(TrainManager.Train Train) {
-			if (Program.CurrentPlatform != Program.Platform.Windows) return;
 			if (!PluginLoaded) return;
 			// prepare vehicle state
 			ATS_VEHICLESTATE State = new ATS_VEHICLESTATE();
-			State.Location = Train.Cars[0].FrontAxle.Follower.TrackPosition;
+			State.Location = Train.Cars[0].FrontAxle.Follower.TrackPosition - Train.Cars[0].FrontAxlePosition + 0.5 * Train.Cars[0].Length;
 			State.Speed = (float)(3.6 * Train.Cars[0].Specs.CurrentPerceivedSpeed);
 			double t = 1000.0 * Game.SecondsSinceMidnight;
 			State.Time = (int)Math.Floor(t - 2073600000.0 * Math.Floor(t / 2073600000.0));
@@ -398,17 +395,12 @@ namespace OpenBve {
 					}
 				}
 			}
-			// debug
-			System.Globalization.CultureInfo Culture = System.Globalization.CultureInfo.InvariantCulture;
-			Game.InfoDebugString = "DLL debug: power=" + Handles.Power.ToString(Culture) + ", brake=" + Handles.Brake.ToString(Culture) + ", reverser=" + Handles.Reverser.ToString(Culture) + ", constspeed=" + Handles.ConstantSpeed.ToString(Culture);
 		}
 		
 		// update signal
 		internal static void UpdateSignal(int Aspect) {
-			if (Program.CurrentPlatform != Program.Platform.Windows) return;
 			if (!PluginLoaded) return;
 			if (Aspect != LastSignalAspect) {
-				//Game.AddMessage("SetSignal (signal=" + Aspect.ToString() + ")", Game.MessageDependency.None, Game.MessageType.Game, Game.MessageColor.Magenta, Game.SecondsSinceMidnight + 5.0);
 				{
 					PluginError = true;
 					SetSignal(Aspect);
@@ -420,7 +412,6 @@ namespace OpenBve {
 		
 		// update beacon
 		internal static void UpdateBeacon(TrainManager.Train Train, TrainManager.TrainPendingTransponder Data) {
-			if (Program.CurrentPlatform != Program.Platform.Windows) return;
 			if (!PluginLoaded) return;
 			ATS_BEACONDATA data = new ATS_BEACONDATA();
 			data.Type = (int)Data.Type;
@@ -469,7 +460,8 @@ namespace OpenBve {
 						data.Signal = Game.Sections[s].Aspects[a].Number;
 					}
 				}
-				data.Distance = (float)(Game.Sections[s].TrackPosition - Train.Cars[0].FrontAxle.Follower.TrackPosition);
+				double p = Train.Cars[0].FrontAxle.Follower.TrackPosition - Train.Cars[0].FrontAxlePosition + 0.5 * Train.Cars[0].Length;
+				data.Distance = (float)(Game.Sections[s].TrackPosition - p);
 			}
 			{
 				PluginError = true;
@@ -480,7 +472,6 @@ namespace OpenBve {
 		
 		// update power
 		internal static void UpdatePower(TrainManager.Train Train) {
-			if (Program.CurrentPlatform != Program.Platform.Windows) return;
 			if (!PluginLoaded) return;
 			int p = Train.Specs.CurrentPowerNotch.Driver;
 			if (p != LastPowerNotch) {
@@ -495,7 +486,6 @@ namespace OpenBve {
 		
 		// update brake
 		internal static void UpdateBrake(TrainManager.Train Train) {
-			if (Program.CurrentPlatform != Program.Platform.Windows) return;
 			if (!PluginLoaded) return;
 			int b;
 			if (Train.Cars[Train.DriverCar].Specs.BrakeType == TrainManager.CarBrakeType.AutomaticAirBrake) {
@@ -523,7 +513,6 @@ namespace OpenBve {
 		
 		// update reverser
 		internal static void UpdateReverser(TrainManager.Train Train) {
-			if (Program.CurrentPlatform != Program.Platform.Windows) return;
 			if (!PluginLoaded) return;
 			int r = Train.Specs.CurrentReverser.Driver;
 			if (r != LastReverserPosition) {
@@ -538,7 +527,6 @@ namespace OpenBve {
 		
 		// update doors
 		internal static void UpdateDoors(bool Closed) {
-			if (Program.CurrentPlatform != Program.Platform.Windows) return;
 			if (!PluginLoaded) return;
 			if (Closed) {
 				PluginError = true;
@@ -553,7 +541,6 @@ namespace OpenBve {
 		
 		// update key
 		internal static void UpdateKey(int AtsKeyCode, bool Down) {
-			if (Program.CurrentPlatform != Program.Platform.Windows) return;
 			if (!PluginLoaded) return;
 			if (Down) {
 				if (!AtsKeyPressed[AtsKeyCode]) {
@@ -578,9 +565,8 @@ namespace OpenBve {
 		
 		// update horn
 		internal static void UpdateHorn(int Horn) {
-			if (Program.CurrentPlatform != Program.Platform.Windows) return;
 			if (!PluginLoaded) return;
-			{ 
+			{
 				PluginError = true;
 				HornBlow(Horn);
 				PluginError = false;
@@ -589,36 +575,46 @@ namespace OpenBve {
 		
 		// load ats config
 		internal static bool LoadAtsConfig(string TrainPath, System.Text.Encoding Encoding, TrainManager.Train Train) {
-			string File = Interface.GetCombinedFileName(TrainPath, "ats.cfg");
-			if (System.IO.File.Exists(File)) {
-				string DllTitle = System.IO.File.ReadAllText(File, Encoding).Trim();
-				string DllFile = Interface.GetCombinedFileName(TrainPath, DllTitle);
-				if (System.IO.File.Exists(DllFile)) {
-					if (Program.CurrentPlatform == Program.Platform.Windows) {
-						if (IntPtr.Size == 4) {
-							PluginLoadState State = LoadPlugin(DllFile, Train);
-							switch (State) {
-								case PluginLoadState.CouldNotLoadDll:
-									Interface.AddMessage(Interface.MessageType.Error, false, "The security system plugin " + DllTitle + " could not be loaded in " + File);
-									return false;
-								case PluginLoadState.InvalidPluginVersion:
-									Interface.AddMessage(Interface.MessageType.Error, false, "The security system plugin " + DllTitle + " is of an unsupported version in " + File);
-									return false;
-								case PluginLoadState.Successful:
-									return true;
-								default:
+			string atsConfigFile = Interface.GetCombinedFileName(TrainPath, "ats.cfg");
+			if (System.IO.File.Exists(atsConfigFile)) {
+				string dllTitle = System.IO.File.ReadAllText(atsConfigFile, Encoding).Trim();
+				if (dllTitle.EndsWith(".dll", StringComparison.OrdinalIgnoreCase)) {
+					string dllFile = Interface.GetCombinedFileName(TrainPath, dllTitle);
+					if (System.IO.File.Exists(dllFile)) {
+						if (Program.CurrentPlatform == Program.Platform.Windows) {
+							if (IntPtr.Size == 4) {
+								if (CheckDllHeader(dllFile)) {
+									PluginLoadState state = LoadPlugin(dllFile, Train);
+									switch (state) {
+										case PluginLoadState.CouldNotLoadDll:
+											Interface.AddMessage(Interface.MessageType.Error, false, "The train plugin " + dllTitle + " could not be loaded in " + atsConfigFile);
+											return false;
+										case PluginLoadState.InvalidPluginVersion:
+											Interface.AddMessage(Interface.MessageType.Error, false, "The train plugin " + dllTitle + " is of an unsupported version in " + atsConfigFile);
+											return false;
+										case PluginLoadState.Successful:
+											return true;
+										default:
+											return false;
+									}
+								} else {
+									Interface.AddMessage(Interface.MessageType.Error, false, "The train plugin " + dllTitle + " is of an unsupported binary format or could not be read in " + atsConfigFile);
 									return false;
+								}
+							} else {
+								Interface.AddMessage(Interface.MessageType.Information, false, "Train plugins are not supported in 64-bit environments. The built-in safety systems will be used for this train, which might not be compatible with the route.");
+								return false;
 							}
 						} else {
-							Interface.AddMessage(Interface.MessageType.Error, false, "Security system plugins are not currently supported when running in 64-bit mode. The built-in security systems will be used for this train, which might not be compatible with the route.");
+							Interface.AddMessage(Interface.MessageType.Information, false, "Train plugins are not supported on operating systems other than Windows. The built-in safety systems will be used for this train, which might not be compatible with the route.");
 							return false;
 						}
 					} else {
-						Interface.AddMessage(Interface.MessageType.Information, false, "Train security system plugins cannot be used on operating systems other than Windows. The built-in security systems will be used for this train, which might not be compatible with the route.");
+						Interface.AddMessage(Interface.MessageType.Error, true, "The train plugin " + dllTitle + " could not be found in " + atsConfigFile);
 						return false;
 					}
 				} else {
-					Interface.AddMessage(Interface.MessageType.Error, true, "The security system plugin " + DllTitle + " could not be found in " + File);
+					Interface.AddMessage(Interface.MessageType.Error, false, "The train plugin " + dllTitle + " must have the extension .dll in " + atsConfigFile);
 					return false;
 				}
 			} else {
@@ -626,5 +622,23 @@ namespace OpenBve {
 			}
 		}
 		
+		// check dll header
+		internal static bool CheckDllHeader(string FileName) {
+			try {
+				bool result;
+				using (System.IO.FileStream stream = new System.IO.FileStream(FileName, System.IO.FileMode.Open, System.IO.FileAccess.Read)) {
+					using (System.IO.BinaryReader reader = new System.IO.BinaryReader(stream)) {
+						byte firstByte = reader.ReadByte();
+						byte secondByte = reader.ReadByte();
+						result = firstByte == 0x4d & secondByte == 0x5a;
+						reader.Close();
+					}
+				}
+				return result;
+			} catch {
+				return false;
+			}
+		}
+		
 	}
 }
\ No newline at end of file
